/* Copyright David Leonard, 2000. All rights reserved. */
/* $Id: main.c,v 1.2 2000/05/21 23:29:22 leonard Exp $ */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "node.h"

const char *argv1;
extern int yydebug;;

int
main(argc, argv)
	int argc;
	char *argv[];
{
	vector_t A;
	FILE **Af;
	sym_t rootsym;
	int i;
	off_t expsize;
	struct stat sb;
	scalar_t result;
	int len;

	if (argc < 2)
		errx(1, "usage: %s expr [files ...]", argv[0]);

	/* Parse expression argument */
	argv1 = argv[1];
	lex_init(argv1);
	yydebug = 0; 
	yyparse();
	lex_finalize();

	/* Optimize the expression tree */
	root = optimize(root);

	/* Open all the input files */
	argc -= 2;
	argv += 2;
	A = new_vector();
	Af = malloc(argc * sizeof(FILE*));
	expsize = 0;
	for (i = 0; i < argc; i++) {
		vector_append(A, 0);
		if ((Af[i] = fopen(argv[i], "rb")) == NULL)
			err(1, "%s", argv[i]);
		if (fstat(fileno(Af[i]), &sb) == -1)
			err(1, "stat %s", argv[i]);
		if (i == 0) {
			expsize = sb.st_size;
			if ((expsize % sizeof(scalar_t)) != 0)
				errx(1, "file size must be multiple of %d",
					sizeof(scalar_t));
		} else if (expsize != sb.st_size)
			errx(1, "%s: wrong size (expected %qd bytes)", 
			    argv[i], expsize);
	}

	/* Construct initial symbol table from the A vector */
	rootsym = sym_enter_vector(A, new_ident("A"), NULL);

	if (argc == 0) {
		/* If no files given, write simple answer to stdout */
		result = eval_scalar(root, rootsym);
		printf("%f\n", result);
	}
	else 
	{
		/* Loop over every input voxel */
		do {
			/* Read a voxel from each file into the 'A' vector */
			for (i = 0; i < argc; i++) {
				len = fread(&A->el[i], 1, sizeof A->el[i],
				    Af[i]);
				if (len == -1)
					err(1, "%s: fread", argv[i]);
				if (len != sizeof(scalar_t))
					errx(1, "%s: fread: short read %d",
					    argv[i], len);
			}

			/* Compute and write result to stdout */
			result = eval_scalar(root, rootsym);
			len = fwrite(&result, 1, sizeof result, stdout);
			if (len == -1)
				err(1, "stdout: fwrite");
			if (len != sizeof result)
				errx(1, "stdout: fwrite: shor write %d", len);
		} while (expsize -= sizeof result);
	}

	/* Clean up */
	sym_leave(rootsym);
	exit(0);
}
