/* $Id: node.h,v 1.4 2000/05/22 00:59:33 leonard Exp $ */

#include "errx.h"

struct node;
struct vector;
struct sym;

typedef float 		scalar_t;
typedef int		ident_t;
typedef struct node	*node_t;
typedef struct vector	*vector_t;
typedef struct sym	*sym_t;

typedef struct {
	enum { SCALAR, VECTOR, NODE } v_type;
	union {
		scalar_t	scalar;
		vector_t	vector;
		node_t		node;
	} value;
	#define v_scalar	value.scalar
	#define v_vector	value.vector
	#define v_node		value.node
} value_t;

#define SCALAR_ROUND(s)	(floorf(s + 0.5))

struct vector {
	int	len;
	value_t *el;		/* array of values */
	int	maxlen;
	int	refcnt;
};

enum nodetype {
	NODETYPE_ADD, 
	NODETYPE_SUB, 
	NODETYPE_MUL, 
	NODETYPE_DIV, 
	NODETYPE_EXP, 
	NODETYPE_INDEX, 
	NODETYPE_SUM, 
	NODETYPE_AVG, 
	NODETYPE_LEN, 
	NODETYPE_IDENT, 
	NODETYPE_REAL, 
	NODETYPE_LET,
	NODETYPE_VEC1,
	NODETYPE_VEC2,
	NODETYPE_GEN,
	NODETYPE_RANGE,
	NODETYPE_LAMBDA,
	NODETYPE_APPLY,
};

#define RANGE_EXACT_UPPER	1
#define RANGE_EXACT_LOWER	2

struct node {
	enum nodetype type;
	node_t expr[3];
	ident_t ident;
	int   flags;
	float real;
	int   pos;
};

ident_t		new_ident(const char *);
const char 	*ident_str(ident_t);
int		ident_is_scalar(ident_t);

node_t		new_node(void);
const char *	node_name(node_t);
node_t		optimize(node_t);

vector_t 	new_vector(void);
void		vector_append(vector_t, value_t *);
void		vector_free(vector_t);
void		value_free(value_t *);

sym_t		sym_enter(value_t *, ident_t, sym_t);
sym_t		sym_leave(sym_t);
int 		sym_lookup(ident_t, sym_t, value_t *);

void 		lex_init(const char *);
void 		lex_finalize(void);

scalar_t	eval_scalar(node_t, sym_t);
void		show_error(int, const char *);

int		yyparse(void);
int		yylex(void);
extern node_t	root;

